package com.vandyo.sentry.core.cases;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.vandyo.sentry.core.dto.ErrStatus;
import com.vandyo.sentry.core.dto.Res;
import com.vandyo.sentry.core.exceptions.CaseException;
import com.vandyo.sentry.core.tools.Signature;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 获取日报接口
 */
public class DayReportDecorator extends CaseDecorator {

    private static final  String dayReportUrl = "/car/api/reprt/reportDay/get";

    public DayReportDecorator(Case aCase) {
        super(aCase);
    }

    @Override
    public Res<Map<String, String>> check() throws CaseException {
        Res<Map<String,String>> oldRes = super.check();
        if (oldRes.getSuccess()) {
            Map<String, String> carMap = oldRes.getData();
            Res<Map<String,String>> newRes = new Res<>();
            newRes.setSId(oldRes.getSId());
            newRes.setUId(oldRes.getUId());

            if (Objects.isNull(carMap) || !carMap.containsKey("cid")){
                newRes.setStatus(ErrStatus.ErrUnexpected);
                newRes.setSuccess(false);
            }else {
                String cid = carMap.get("cid");
                Map<String,String> paramMap = new HashMap<>();
                paramMap.put("cid",cid);
                paramMap.put("rptver",String.valueOf(System.currentTimeMillis()/1000 - 60*60*24));

                Map<String,String> signMap= new HashMap<>();
                signMap.put("uid",oldRes.getUId());
                signMap.put("Data",JSONUtil.parseFromMap(paramMap).toString());
                Map<String,String> signedMap = Signature.sign(signMap,oldRes.getSId());
                signedMap.remove("Data");
                try {
                    HttpResponse response = HttpRequest.post(Signature.host + dayReportUrl + "?" + Signature.getUrlParamsByMap(signedMap))
                            .body(JSONUtil.parseFromMap(paramMap).toString())
                            .execute();
                    if (response.isOk()) {
                        JSONObject jsonObject = JSONUtil.parseObj(response.body());
                        //todo 检测 返回值可用性
                        newRes.setData(carMap);
                        newRes.setSuccess(true);
                    } else {
                        newRes.setKey(dayReportUrl);
                        newRes.setStatus(Signature.matchStatus(response.getStatus()));
                        newRes.setSuccess(false);
                    }
                }catch (Exception e){
                    if (e.getMessage().toLowerCase().contains("timeout")){
                        throw new CaseException(dayReportUrl, ErrStatus.ErrTimeout);
                    }else {
                        throw new CaseException(dayReportUrl,ErrStatus.ErrUnknown);
                    }
                }
            }
            return newRes;
        }else {
            oldRes.setIgnore(true);
            return oldRes;
        }
    }
}
